package cn.kuaiyun.storage;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileInputStream;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.io.OutputStream;
import java.net.HttpURLConnection;
import java.net.URL;
import java.util.HashMap;
import java.util.Map;
import java.util.Map.Entry;

import sun.misc.BASE64Encoder;
import net.sf.json.JSONObject;

/**
 * 
 * 快云存储API使用demo
 * 
 * @author HAPPY
 * 
 */
public class StorageAPIDemo {

	/**
	 * 接口API地址
	 */
	public static final String API = "http://api.storagesdk.com/";

	/**
	 * 接口方法
	 */
	public static final String METHOD_GETTOKEN = "restful/storageapi/storage/getToken"; // 获取操作密钥token
	public static final String METHOD_UPLOAD = "restful/storageapi/file/uploadFile"; // 上传文件
	public static final String METHOD_GETURL = "restful/storageapi/file/getFileUrl"; // 获取文件访问链接
	public static final String METHOD_DELFILE = "restful/storageapi/file/deleteFile"; // 删除文件
	public static final String METHOD_CREATE_FOLDER = "restful/storageapi/folder/createFolder";// 创建文件夹
	public static final String METHOD_DELETE_FOLDER = "restful/storageapi/folder/deleteFolder"; // 删除文件夹
	public static final String METHOD_SUB_FOLDERANDFILES = "restful/storageapi/folder/getSubFoldersAndFiles"; // 获取当前目录下子目录及其文件
	public static final String METHOD_RENAME_FOLDER = "restful/storageapi/folder/renameFolder"; // 重命名文件夹
	public static final String METHOD_RENAME_FILE = "restful/storageapi/file/renameFile"; // 重命名文件
	public static final String METHOD_COPYFILE = "restful/storageapi/file/copyFile"; // 复制文件
	public static final String METHOD_COPYFOLDER = "restful/storageapi/folder/copyFolder"; // 复制文件夹
	
	
	/**
	 * 参数，此处为测试账号业务信息； 替换为用户的对象存储业务信息，在会员中心可以获取
	 */
	public static final String ACCESS_KEY = "I1I7CLU4708LI8XQ0ODL"; // 用户key值对：Access_Key，可在会员中心获取
	public static final String SECRET_KEY = "N41hrYy14gI03RpnZijfezGl/VUtZP3b77vjF6nn"; // 用户key值对：Secret_Key，可在会员中心获取
	public static final String RESOURCE = "VG3OQ81wtrNGANeuq8IdwWxGFK0xdA3X"; // API调用来源，可在会员中心点击获取
	public static final String VOUCHER = "dcb99d196dd2418ec1fdfa7269ee4b81"; // 凭证，用户通过accesskey和secretkey获取的，可在会员中心点击，以邮件形式获取
	//
	public static String token = ""; // 用户操作秘钥，是用户调用api接口时必须用到的的秘钥；可以通过API获取

	/**
	 * demo测试
	 * 
	 * @param args
	 */
	public static void main(String[] args) {
		// 获取token
		String tokenMsg = getToken();
		System.out.println("获取token的返回结果:" + tokenMsg);
		token = JSONObject.fromObject(tokenMsg).get("message").toString().split(":")[1];

		// 测试空间名称，已经创建的空间名
		String bucketName = "demobkt"; // 用户的空间名称
		
		// 上传文件
		String localFile = "C:\\demo.png"; // 本地图片路径
		String fileName = "demo.png"; // 上传到对象存储中文件路径名；如果是多级的，自动创建文件夹，例folder/demo.png，自动创建folder目录，并在该目录下上传demo.png文件
		String uploadInfo = uploadFile(bucketName, localFile, fileName);
		System.out.println("上传文件返回结果:" + uploadInfo);

		// 获取文件访问链接
		int minutes = 0; // 获取链接时效,可不传；默认是0，无时效，即永久有效。
		String objectUrlInfo = getFileUrl(bucketName, fileName, minutes);
		System.out.println("获取文件访问链接的返回结果:" + objectUrlInfo);

		// 删除文件
		String delFileInfo = delFile(bucketName, fileName);
		System.out.println("删除文件的返回结果:" + delFileInfo);
		
		// 创建文件夹
		String fileFolder = "demofd0"; // 文件夹已经存在，创建失败
		String createFolder  =  createFolder(bucketName, fileFolder);
		System.out.println("创建文件夹的返回结果:" + createFolder);
			
		// 删除文件夹
		String delfolder = delFolder(bucketName, fileFolder);
		System.out.println("删除文件夹的返回结果:" + delfolder);

		// 获取当前文件目录下子目录列表及文件列表
		String folderName = "demofd"; // 文件目录名
		Boolean isGetFile = true; // 是否获取文件[true.是,false.否]
		String getSubFoldersAndFiles = getSubFoldersAndFiles(bucketName, folderName, isGetFile);
		System.out.println("获取当前目录下子目录及文件的返回结果:" + getSubFoldersAndFiles);

		// 重命名文件
		String oldFileName = "demofd/000000.png"; // 原文件名称，已经存在的文件名
		String newFileName = "demofd/111111.png"; // 新文件名称，如果已经有此文件存在，重命名失败
		String renameFile = renameFile(bucketName, oldFileName, newFileName);
		System.out.println("重命名文件的返回结果:" + renameFile);

		// 重命名文件夹
		String oldFolderName = "demofd/folder1"; // 原文件夹名称，已经存在的目录文件夹（根目录，即空间目录不能修改）
		String newFolderName = "demofd/folder2"; // 新文件夹名称，如果已经有此文件夹存在，重命名失败
		String renameFolder = renameFolder(bucketName, oldFolderName, newFolderName);
		System.out.println("重命名文件夹的返回结果:" + renameFolder);

		// 复制文件
		String newBucketName = "demobkt"; // 目标空间，必须是创建过的空间名，可以和源空间名一样
		String newFileName2 = "floder/000000.png"; // 复制后的文件名，若复制后的文件夹不存在，默认创建；若复制后的文件存在，默认覆盖
		String copyFile = copyFile(bucketName, newFileName, newBucketName, newFileName2);
		System.out.println("复制文件的返回结果:" + copyFile);

		// 复制文件夹
		String newFolderName2 = "folder1"; // 复制后的文件夹名，若复制后的文件夹不存在，默认创建；若复制后的文件夹存在，默认合并
		String copyFolder = copyFolder(bucketName, newFolderName, newBucketName, newFolderName2);
		System.out.println("复制文件夹的返回结果:" + copyFolder);
	}

	/**
	 * 获取操作密钥token
	 */
	public static String getToken() {
		JSONObject jsStr = new JSONObject();
		jsStr.put("voucher", VOUCHER);
		jsStr.put("accessKey", ACCESS_KEY);
		jsStr.put("secretKey", SECRET_KEY);
		jsStr.put("resource", RESOURCE);
		String msg = sendPost(API + METHOD_GETTOKEN, jsStr.toString(), null, null);
		return msg;
	}

	/**
	 * 上传文件
	 * 
	 * @return
	 */
	public static String uploadFile(String bucketName, String localFile, String fileName) {
		try {
			File file = new File(localFile);
			InputStream in = null;
			try {
				in = new FileInputStream(file);
			} catch (Exception e) {
				e.printStackTrace();
			}
			return uploadFileStream(bucketName, in, fileName);
		} catch (Exception e) {
			e.printStackTrace();
		}
		return null;
	}

	/**
	 * 上传文件流
	 * 
	 * @return
	 */
	public static String uploadFileStream(String bucketName, InputStream inFileStream, String fileName) {
		try {
			BASE64Encoder encoder = new BASE64Encoder();
			fileName = encoder.encode(fileName.getBytes("utf-8"));

			Map<String, String> hander = new HashMap<String, String>();
			hander.put("token", token);
			hander.put("resource", RESOURCE);
			hander.put("bucketName", bucketName);
			hander.put("fileName", fileName);
			hander.put("length", inFileStream.available() + "");

			return sendPost(API + METHOD_UPLOAD, null, inFileStream, hander);
		} catch (Exception e) {
			e.printStackTrace();
		}
		return null;
	}

	/**
	 * 获取文件访问链接
	 * 
	 * @return
	 */
	public static String getFileUrl(String bucketName, String keyName, int minutes) {
		try {
			JSONObject jsStr = new JSONObject();
			jsStr.put("token", token);
			jsStr.put("resource", RESOURCE);
			jsStr.put("bucketName", bucketName);
			jsStr.put("fileName", keyName); // 拼接文件路径
			jsStr.put("minutes", minutes);// 可以不传，默认是无时效。
			return sendPost(API + METHOD_GETURL, jsStr.toString(), null, null);
		} catch (Exception e) {
			e.printStackTrace();
		}
		return null;
	}

	/**
	 * 删除文件
	 * 
	 * @return
	 */
	public static String delFile(String bucketName, String delFileName) {
		try {
			JSONObject jsStr = new JSONObject();
			jsStr.put("token", token);
			jsStr.put("resource", RESOURCE);
			jsStr.put("bucketName", bucketName);
			jsStr.put("fileName", delFileName);
			return sendPost(API + METHOD_DELFILE, jsStr.toString(), null, null);
		} catch (Exception e) {
			e.printStackTrace();
		}
		return null;
	}

	/**
	 * 创建文件夹
	 * @return
	 */
	public static String createFolder(String bucketName, String folderName){
		try {
			JSONObject jsStr = new JSONObject();
			jsStr.put("bucketName",	bucketName);
			jsStr.put("folderName", folderName);
			jsStr.put("token", token);
			jsStr.put("resource", RESOURCE);
			return sendPost(API + METHOD_CREATE_FOLDER , jsStr.toString(), null, null);
		} catch (Exception e) {
			e.printStackTrace();
		}
		return null;
	}

	/**
	 * 获取当前目录下子目录及文件
	 * 
	 * @return
	 */
	public static String getSubFoldersAndFiles(String bucketName, String folderName, boolean isGetFile) {
		try {
			JSONObject jsStr = new JSONObject();
			jsStr.put("resource", RESOURCE);
			jsStr.put("token", token);
			jsStr.put("bucketName", bucketName);
			jsStr.put("folderName", folderName);
			jsStr.put("isGetFile", isGetFile);
			return sendPost(API + METHOD_SUB_FOLDERANDFILES, jsStr.toString(), null, null);
		} catch (Exception e) {
			e.printStackTrace();
		}
		return null;
	}
	
	/**
	 * 重命名文件
	 * 
	 * @return
	 */
	public static String renameFile(String bucketName, String oldFileName, String newFileName) {
		try {
			JSONObject jsStr = new JSONObject();
			jsStr.put("token", token);
			jsStr.put("bucketName", bucketName);
			jsStr.put("resource", RESOURCE);
			jsStr.put("oldFileName", oldFileName);
			jsStr.put("newFileName", newFileName);
			return sendPost(API + METHOD_RENAME_FILE, jsStr.toString(), null, null);
		} catch (Exception e) {
			e.printStackTrace();
		}
		return null;
	}

	/**
	 * 重命名文件夹
	 * 
	 * @return
	 */
	public static String renameFolder(String bucketName, String oldFolderName, String newFolderName) {
		try {
			JSONObject jsStr = new JSONObject();
			jsStr.put("token", token);
			jsStr.put("bucketName", bucketName);
			jsStr.put("resource", RESOURCE);
			jsStr.put("oldFolderName", oldFolderName);
			jsStr.put("newFolderName", newFolderName);
			return sendPost(API + METHOD_RENAME_FOLDER, jsStr.toString(), null, null);
		} catch (Exception e) {
			e.printStackTrace();
		}
		return null;
	}

	/**
	 * 复制文件
	 * 
	 * @return
	 */
	public static String copyFile(String oldBucketName, String oldFileName, String newBucketName, String newFileName) {
		try {
			JSONObject jsStr = new JSONObject();
			jsStr.put("token", token);
			jsStr.put("resource", RESOURCE);
			jsStr.put("oldBucketName", oldBucketName);
			jsStr.put("oldFileName", oldFileName);
			jsStr.put("newBucketName", newBucketName);
			jsStr.put("newFileName", newFileName);
			return sendPost(API + METHOD_COPYFILE, jsStr.toString(), null, null);
		} catch (Exception e) {
			e.printStackTrace();
		}
		return null;
	}

	/**
	 * 复制文件夹
	 * 
	 * @return
	 */
	public static String copyFolder(String oldBucketName, String oldFolderName, String newBucketName,
			String newFolderName) {
		try {
			JSONObject jsStr = new JSONObject();
			jsStr.put("token", token);
			jsStr.put("resource", RESOURCE);
			jsStr.put("oldBucketName", oldBucketName);
			jsStr.put("oldFolderName", oldFolderName);
			jsStr.put("newBucketName", newBucketName);
			jsStr.put("newFolderName", newFolderName);
			return sendPost(API + METHOD_COPYFOLDER, jsStr.toString(), null, null);
		} catch (Exception e) {
			e.printStackTrace();
		}
		return null;
	}

	/**
	 * 删除文件夹
	 * @return
	 */
	public static String delFolder(String bucketName, String folderName){
		try {
			JSONObject jsStr = new JSONObject();
			jsStr.put("token", token);
			jsStr.put("bucketName", bucketName);
			jsStr.put("resource", RESOURCE);
			jsStr.put("folderName", folderName);
			return sendPost(API + METHOD_DELETE_FOLDER , jsStr.toString(), null, null);
		} catch (Exception e) {
			e.printStackTrace();
		}
		return null;
	}
	
	/**
	 * HTTP请求块（POST）
	 * 
	 * @param url
	 *            请求链接
	 * @param JsonString
	 *            请求体中的参数
	 * @param input
	 *            请求文件流
	 * @param hander
	 *            请求头里的参数
	 * @return
	 */
	private static String sendPost(String url, String JsonString, InputStream input, Map<String, String> hander) {
		String result = "";
		// 创建连接
		try {
			URL realUrl = new URL(url);
			// 打开和URL之间的连接
			HttpURLConnection conn = (HttpURLConnection) realUrl.openConnection();
			// 设置通用的请求属性
			conn.setRequestProperty("Content-Type", " application/json");// 设定请求格式,json，也可以设定xml格式的
			conn.setRequestProperty("Accept-Charset", "UTF-8"); // 设置编码语言
			conn.setRequestProperty("X-Auth-Token", "token"); // 设置请求的token
			conn.setRequestProperty("Connection", "keep-alive"); // 设置连接的状态
			conn.setRequestProperty("Transfer-Encoding", "chunked");// 设置传输编码
			// post请求，无缓存
			conn.setRequestMethod("POST"); // 可以根据需要提交
			conn.setUseCaches(false);// 设置缓存，注意设置请求方法为post不能用缓存
			conn.setChunkedStreamingMode(0);// 设置该选项，则不使用HttpURLConnection的缓存机制，直接将流提交到服务器上
			conn.setInstanceFollowRedirects(true);
			// 发送POST请求必须设置如下两行
			conn.setDoOutput(true);
			conn.setDoInput(true);

			// 请求参数在请求头中
			if (hander != null) {
				for (Entry<String, String> entry : hander.entrySet()) {
					conn.addRequestProperty(entry.getKey(), entry.getValue());
				}
			}
			// 建立实际的连接
			conn.connect();

			int code = 0;
			//
			if (JsonString != null) {
				OutputStream out = conn.getOutputStream();
				out.write(JsonString.getBytes());
				code = conn.getResponseCode();
				out.flush();
				out.close();
			}

			if (input != null) {
				OutputStream out = conn.getOutputStream();
				byte[] bytes = new byte[1024];
				int len = -1;
				while ((len = input.read(bytes)) != -1) {
					out.write(bytes, 0, len);
				}
				code = conn.getResponseCode();
				input.close();
				out.flush();
				out.close();
			}

			// 读取响应
			if (code == 200) {
				BufferedReader read = new BufferedReader(new InputStreamReader(conn.getInputStream(), "UTF-8"));
				StringBuffer stb = new StringBuffer();
				String line = "";
				while ((line = read.readLine()) != null) {
					stb.append(line);
				}
				read.close();
				String message = stb.toString();
				if (message != null && !"".equals(message)) {
					JSONObject returnObject = JSONObject.fromObject(message);
					result = returnObject.toString();
				} else {
				}
			} else {
				result = conn.getResponseMessage();
			}

			// 断开连接
			conn.disconnect();

		} catch (Exception e) {
			e.printStackTrace();
			result = "调用post方法异常，异常原因：" + e.getMessage();
		}
		return result;
	}
}
